# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["GeofenceUpdateParams", "Circle", "CircleCenter", "Isochrone", "Polygon", "PolygonGeojson"]


class GeofenceUpdateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    circle: Circle
    """Use this object to update details of a circular geofence.

    Please note that this object is mandatory only when `type` is `circle`. When the
    `type` is not `circle`, the properties of this object will be ignored while
    creating the geofence.
    """

    isochrone: Isochrone
    """Use this object to update details of an isochrone based geofence.

    Please note that this object is mandatory only when `type` is `isochrone`. When
    the `type` is not `isochrone`, the properties of this object will be ignored
    while creating the geofence.
    """

    meta_data: object
    """Updated the `meta_data` associated with a geofence.

    Use this field to define custom attributes that provide more context and
    information about the geofence being updated like country, group ID etc.

    The data being added should be in valid JSON object format (i.e. `key` and
    `value` pairs). Max size allowed for the object is 65kb.
    """

    name: str
    """Use this parameter to update the `name` of a geofence.

    Users can assign meaningful custom names to their geofences.
    """

    polygon: Polygon
    """Use this object to update details of a custom polygon geofence.

    Please note that this object is mandatory only when `type` is `polygon`. When
    the `type` is not `polygon`, the properties of this object will be ignored while
    creating the geofence.

    Self-intersecting polygons or polygons containing other polygons are invalid and
    will be removed while processing the request.

    Area of the polygon should be less than 2000 km<sup>2</sup>.
    """

    tags: List[str]
    """Use this parameter to add/modify one or multiple `tags` of a geofence.

    `tags` can be used to search or filter geofences (using `Get Geofence List`
    method).

    Valid values for updating `tags` consist of alphanumeric characters (A-Z, a-z,
    0-9) along with the underscore ('\\__') and hyphen ('-') symbols.
    """

    type: Literal["`circle`", "`polygon`", "`isochrone`"]
    """Use this parameter to update the `type` of a geofence.

    Please note that you will need to provide required details for creating a
    geofence of the new `type`. Check other parameters of this method to know more.
    """


class CircleCenter(TypedDict, total=False):
    lat: float
    """Latitude of the `center` location."""

    lon: float
    """Longitude of the `center` location."""


class Circle(TypedDict, total=False):
    center: Required[CircleCenter]
    """
    Use this parameter to update the coordinate of the location which will act as
    the center of a circular geofence.
    """

    radius: float
    """Use this parameter to update the radius of the circular geofence, in meters.

    Maximum value allowed is 50000 meters.
    """


class Isochrone(TypedDict, total=False):
    contours_meter: int
    """
    Use this parameter to update the distance, in meters, for which an isochrone
    polygon needs to be determined. When provided, the API would create a geofence
    representing the area that can be reached after driving the given number of
    meters starting from the point specified in `coordinates`.

    The maximum distance that can be specified is 60000 meters (60km).

    At least one of `contours_meter` or `contours_minute` is mandatory when `type`
    is `isochrone`.
    """

    contours_minute: int
    """
    Use this parameter to update the duration, in minutes, for which an isochrone
    polygon needs to be determined. When provided, the API would create a geofence
    representing the area that can be reached after driving for the given number of
    minutes starting from the point specified in `coordinates`.

    The maximum duration that can be specified is 40 minutes.

    At least one of `contours_meter` or `contours_minute` is mandatory when `type`
    is `isochrone`.
    """

    coordinates: str
    """
    Use this parameter to update the coordinates of the location, in
    [latitude,longitude] format, which would act as the starting point for
    identifying the isochrone polygon or the boundary of reachable area.
    """

    denoise: float
    """
    A floating point value from 0.0 to 1.0 that can be used to remove smaller
    contours. A value of 1.0 will only return the largest contour for a given value.
    A value of 0.5 drops any contours that are less than half the area of the
    largest contour in the set of contours for that same value.

    Use this parameter to update the denoise value of the isochrone geofence.
    """

    departure_time: int
    """
    Use this parameter to update the `departure_time`, expressed as UNIX epoch
    timestamp in seconds. The isochrone boundary will be determined based on the
    typical traffic conditions at the given time.

    If no input is provided for this parameter then, the traffic conditions at the
    time of making the request are considered by default. Please note that because
    of this behavior the geofence boundaries may change even if the `departure_time`
    was not specifically provided at the time of updating the geofence.
    """

    mode: str
    """
    Use this parameter to update the driving mode that the service should use to
    determine the isochrone line. For example, if you use `car`, the API will return
    an isochrone polygon that a car can cover within the specified time or after
    driving the specified distance. Using `truck` will return an isochrone that a
    truck can reach after taking into account appropriate truck routing
    restrictions.
    """


class PolygonGeojson(TypedDict, total=False):
    geometry: Iterable[Iterable[float]]
    """
    An array of coordinates in the [longitude, latitude] format, representing the
    geofence boundary.
    """

    type: str
    """Type of the geoJSON geometry. Should always be `Polygon`."""


class Polygon(TypedDict, total=False):
    geojson: PolygonGeojson
    """An object to collect geoJSON details of the `polygon` geofence.

    The contents of this object follow the
    [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .json import (
    JsonResource,
    AsyncJsonResource,
    JsonResourceWithRawResponse,
    AsyncJsonResourceWithRawResponse,
    JsonResourceWithStreamingResponse,
    AsyncJsonResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["DistancematrixResource", "AsyncDistancematrixResource"]


class DistancematrixResource(SyncAPIResource):
    @cached_property
    def json(self) -> JsonResource:
        return JsonResource(self._client)

    @cached_property
    def with_raw_response(self) -> DistancematrixResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return DistancematrixResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DistancematrixResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return DistancematrixResourceWithStreamingResponse(self)


class AsyncDistancematrixResource(AsyncAPIResource):
    @cached_property
    def json(self) -> AsyncJsonResource:
        return AsyncJsonResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDistancematrixResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDistancematrixResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDistancematrixResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncDistancematrixResourceWithStreamingResponse(self)


class DistancematrixResourceWithRawResponse:
    def __init__(self, distancematrix: DistancematrixResource) -> None:
        self._distancematrix = distancematrix

    @cached_property
    def json(self) -> JsonResourceWithRawResponse:
        return JsonResourceWithRawResponse(self._distancematrix.json)


class AsyncDistancematrixResourceWithRawResponse:
    def __init__(self, distancematrix: AsyncDistancematrixResource) -> None:
        self._distancematrix = distancematrix

    @cached_property
    def json(self) -> AsyncJsonResourceWithRawResponse:
        return AsyncJsonResourceWithRawResponse(self._distancematrix.json)


class DistancematrixResourceWithStreamingResponse:
    def __init__(self, distancematrix: DistancematrixResource) -> None:
        self._distancematrix = distancematrix

    @cached_property
    def json(self) -> JsonResourceWithStreamingResponse:
        return JsonResourceWithStreamingResponse(self._distancematrix.json)


class AsyncDistancematrixResourceWithStreamingResponse:
    def __init__(self, distancematrix: AsyncDistancematrixResource) -> None:
        self._distancematrix = distancematrix

    @cached_property
    def json(self) -> AsyncJsonResourceWithStreamingResponse:
        return AsyncJsonResourceWithStreamingResponse(self._distancematrix.json)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from ..geofence_entity_create_param import GeofenceEntityCreateParam

__all__ = ["BatchCreateParams"]


class BatchCreateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    geofences: Iterable[GeofenceEntityCreateParam]
    """
    An array of objects to collect the details of the multiple geofences that need
    to be created.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AssetListParams"]


class AssetListParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    cluster: Literal["america"]
    """the cluster of the region you want to use"""

    include_all_of_attributes: str
    """Use this parameter to filter the assets by their `attributes`.

    Only the assets having all the `attributes` added to this parameter, will be
    returned in the response. Multiple `attributes` can be separated using pipes
    (`|`).

    Please note the attributes are case sensitive. Also, this parameter can not be
    used in conjunction with `include_any_of_attributes` parameter.
    """

    include_any_of_attributes: str
    """Use this parameter to filter the assets by their `attributes`.

    Assets having at least one of the `attributes` added to this parameter, will be
    returned in the response. Multiple `attributes` can be separated using pipes
    (`|`).

    Please note the attributes are case sensitive. Also, this parameter can not be
    used in conjunction with `include_all_of_attributes` parameter.
    """

    pn: int
    """Denotes page number.

    Use this along with the `ps` parameter to implement pagination for your searched
    results. This parameter does not have a maximum limit but would return an empty
    response in case a higher value is provided when the result-set itself is
    smaller.
    """

    ps: int
    """Denotes number of search results per page.

    Use this along with the `pn` parameter to implement pagination for your searched
    results.
    """

    sort: str
    """Provide a single field to sort the results by.

    Only `updated_at` or `created_at` fields can be selected for ordering the
    results.

    By default, the result is sorted by `created_at` field in the descending order.
    Allowed values for specifying the order are `asc` for ascending order and `desc`
    for descending order.
    """

    tags: str
    """
    **This parameter will be deprecated soon! Please use the
    `include_all_of_attributes` or `include_any_of_attributes` parameters to provide
    labels or markers for the assets to be retrieved.**

    `tags` can be used to filter the assets. Only those assets which have all the
    `tags` provided, will be included in the result. In case multiple `tags` need to
    be specified, use `,` to separate them.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.multigeocode import (
    PlaceCreateResponse,
    PlaceDeleteResponse,
    PlaceUpdateResponse,
    PlaceRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPlace:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: NextbillionSDK) -> None:
        place = client.multigeocode.place.create(
            key="key=API_KEY",
            place=[{"geopoint": {}}],
        )
        assert_matches_type(PlaceCreateResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: NextbillionSDK) -> None:
        place = client.multigeocode.place.create(
            key="key=API_KEY",
            place=[
                {
                    "geopoint": {
                        "lat": 0,
                        "lng": 0,
                    },
                    "address": "address",
                    "building": "building",
                    "city": "city",
                    "country": '"country":"IND"',
                    "district": "district",
                    "house": "house",
                    "poi": {"title": "title"},
                    "postal_code": "postalCode",
                    "state": "state",
                    "street": "street",
                    "sub_district": "subDistrict",
                }
            ],
            data_source={
                "ref_id": "refId",
                "source": "source",
                "status": "enable",
            },
            force=True,
            score=0,
        )
        assert_matches_type(PlaceCreateResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: NextbillionSDK) -> None:
        response = client.multigeocode.place.with_raw_response.create(
            key="key=API_KEY",
            place=[{"geopoint": {}}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        place = response.parse()
        assert_matches_type(PlaceCreateResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: NextbillionSDK) -> None:
        with client.multigeocode.place.with_streaming_response.create(
            key="key=API_KEY",
            place=[{"geopoint": {}}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            place = response.parse()
            assert_matches_type(PlaceCreateResponse, place, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: NextbillionSDK) -> None:
        place = client.multigeocode.place.retrieve(
            doc_id="docId",
            key="key=API_KEY",
        )
        assert_matches_type(PlaceRetrieveResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: NextbillionSDK) -> None:
        response = client.multigeocode.place.with_raw_response.retrieve(
            doc_id="docId",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        place = response.parse()
        assert_matches_type(PlaceRetrieveResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: NextbillionSDK) -> None:
        with client.multigeocode.place.with_streaming_response.retrieve(
            doc_id="docId",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            place = response.parse()
            assert_matches_type(PlaceRetrieveResponse, place, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            client.multigeocode.place.with_raw_response.retrieve(
                doc_id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: NextbillionSDK) -> None:
        place = client.multigeocode.place.update(
            doc_id="docId",
            key="key=API_KEY",
        )
        assert_matches_type(PlaceUpdateResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: NextbillionSDK) -> None:
        place = client.multigeocode.place.update(
            doc_id="docId",
            key="key=API_KEY",
            data_source={
                "ref_id": "refId",
                "source": "source",
                "status": "enable",
            },
            place=[
                {
                    "address": "address",
                    "building": "building",
                    "city": "city",
                    "country": "country",
                    "district": "district",
                    "geopoint": {
                        "lat": 0,
                        "lng": 0,
                    },
                    "house": "house",
                    "poi": {"title": "title"},
                    "postal_code": "postalCode",
                    "state": "state",
                    "street": "street",
                    "sub_district": "subDistrict",
                }
            ],
            score=0,
        )
        assert_matches_type(PlaceUpdateResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: NextbillionSDK) -> None:
        response = client.multigeocode.place.with_raw_response.update(
            doc_id="docId",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        place = response.parse()
        assert_matches_type(PlaceUpdateResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: NextbillionSDK) -> None:
        with client.multigeocode.place.with_streaming_response.update(
            doc_id="docId",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            place = response.parse()
            assert_matches_type(PlaceUpdateResponse, place, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            client.multigeocode.place.with_raw_response.update(
                doc_id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: NextbillionSDK) -> None:
        place = client.multigeocode.place.delete(
            doc_id="docId",
            key="key=API_KEY",
        )
        assert_matches_type(PlaceDeleteResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: NextbillionSDK) -> None:
        response = client.multigeocode.place.with_raw_response.delete(
            doc_id="docId",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        place = response.parse()
        assert_matches_type(PlaceDeleteResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: NextbillionSDK) -> None:
        with client.multigeocode.place.with_streaming_response.delete(
            doc_id="docId",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            place = response.parse()
            assert_matches_type(PlaceDeleteResponse, place, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_delete(self, client: NextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            client.multigeocode.place.with_raw_response.delete(
                doc_id="",
                key="key=API_KEY",
            )


class TestAsyncPlace:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncNextbillionSDK) -> None:
        place = await async_client.multigeocode.place.create(
            key="key=API_KEY",
            place=[{"geopoint": {}}],
        )
        assert_matches_type(PlaceCreateResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        place = await async_client.multigeocode.place.create(
            key="key=API_KEY",
            place=[
                {
                    "geopoint": {
                        "lat": 0,
                        "lng": 0,
                    },
                    "address": "address",
                    "building": "building",
                    "city": "city",
                    "country": '"country":"IND"',
                    "district": "district",
                    "house": "house",
                    "poi": {"title": "title"},
                    "postal_code": "postalCode",
                    "state": "state",
                    "street": "street",
                    "sub_district": "subDistrict",
                }
            ],
            data_source={
                "ref_id": "refId",
                "source": "source",
                "status": "enable",
            },
            force=True,
            score=0,
        )
        assert_matches_type(PlaceCreateResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.multigeocode.place.with_raw_response.create(
            key="key=API_KEY",
            place=[{"geopoint": {}}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        place = await response.parse()
        assert_matches_type(PlaceCreateResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.multigeocode.place.with_streaming_response.create(
            key="key=API_KEY",
            place=[{"geopoint": {}}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            place = await response.parse()
            assert_matches_type(PlaceCreateResponse, place, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        place = await async_client.multigeocode.place.retrieve(
            doc_id="docId",
            key="key=API_KEY",
        )
        assert_matches_type(PlaceRetrieveResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.multigeocode.place.with_raw_response.retrieve(
            doc_id="docId",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        place = await response.parse()
        assert_matches_type(PlaceRetrieveResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.multigeocode.place.with_streaming_response.retrieve(
            doc_id="docId",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            place = await response.parse()
            assert_matches_type(PlaceRetrieveResponse, place, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            await async_client.multigeocode.place.with_raw_response.retrieve(
                doc_id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncNextbillionSDK) -> None:
        place = await async_client.multigeocode.place.update(
            doc_id="docId",
            key="key=API_KEY",
        )
        assert_matches_type(PlaceUpdateResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        place = await async_client.multigeocode.place.update(
            doc_id="docId",
            key="key=API_KEY",
            data_source={
                "ref_id": "refId",
                "source": "source",
                "status": "enable",
            },
            place=[
                {
                    "address": "address",
                    "building": "building",
                    "city": "city",
                    "country": "country",
                    "district": "district",
                    "geopoint": {
                        "lat": 0,
                        "lng": 0,
                    },
                    "house": "house",
                    "poi": {"title": "title"},
                    "postal_code": "postalCode",
                    "state": "state",
                    "street": "street",
                    "sub_district": "subDistrict",
                }
            ],
            score=0,
        )
        assert_matches_type(PlaceUpdateResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.multigeocode.place.with_raw_response.update(
            doc_id="docId",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        place = await response.parse()
        assert_matches_type(PlaceUpdateResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.multigeocode.place.with_streaming_response.update(
            doc_id="docId",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            place = await response.parse()
            assert_matches_type(PlaceUpdateResponse, place, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            await async_client.multigeocode.place.with_raw_response.update(
                doc_id="",
                key="key=API_KEY",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncNextbillionSDK) -> None:
        place = await async_client.multigeocode.place.delete(
            doc_id="docId",
            key="key=API_KEY",
        )
        assert_matches_type(PlaceDeleteResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.multigeocode.place.with_raw_response.delete(
            doc_id="docId",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        place = await response.parse()
        assert_matches_type(PlaceDeleteResponse, place, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.multigeocode.place.with_streaming_response.delete(
            doc_id="docId",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            place = await response.parse()
            assert_matches_type(PlaceDeleteResponse, place, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncNextbillionSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            await async_client.multigeocode.place.with_raw_response.delete(
                doc_id="",
                key="key=API_KEY",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.skynet import (
    trip_end_params,
    trip_start_params,
    trip_delete_params,
    trip_update_params,
    trip_retrieve_params,
    trip_get_summary_params,
)
from ...types.skynet.simple_resp import SimpleResp
from ...types.skynet.trip_start_response import TripStartResponse
from ...types.skynet.trip_retrieve_response import TripRetrieveResponse
from ...types.skynet.trip_get_summary_response import TripGetSummaryResponse

__all__ = ["TripResource", "AsyncTripResource"]


class TripResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TripResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return TripResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TripResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return TripResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TripRetrieveResponse:
        """
        Retrieves detailed information about a specific trip.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/skynet/trip/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    trip_retrieve_params.TripRetrieveParams,
                ),
            ),
            cast_to=TripRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        key: str,
        asset_id: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        attributes: object | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        meta_data: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        stops: Iterable[trip_update_params.Stop] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Updates the data of a specified trip with the provided data.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          asset_id: Use this param to update the ID of the asset which made this trip. Please be
              cautious when using this field as providing an ID other than what was provided
              at the time of starting the trip, will link a new asset to the trip and un-link
              the original asset, even if the trip is still active.

          cluster: the cluster of the region you want to use

          attributes: Use this field to update the `attributes` of the trip. Please note that when
              updating the `attributes` field, previously added information will be
              overwritten.

          description: Use this parameter to update the custom description of the trip.

          meta_data: Use this JSON object to update additional details about the trip. This property
              is used to add any custom information / context about the trip.

              Please note that updating the `meta_data` field will overwrite the previously
              added information.

          name: Use this property to update the name of the trip.

          stops: Use this object to update the details of the stops made during the trip. Each
              object represents a single stop.

              Please note that when updating this field, the new stops will overwrite any
              existing stops configured for the trip.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/skynet/trip/{id}",
            body=maybe_transform(
                {
                    "asset_id": asset_id,
                    "attributes": attributes,
                    "description": description,
                    "meta_data": meta_data,
                    "name": name,
                    "stops": stops,
                },
                trip_update_params.TripUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    trip_update_params.TripUpdateParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    def delete(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Deletes a specified trip from the system.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/skynet/trip/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    trip_delete_params.TripDeleteParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    def end(
        self,
        *,
        key: str,
        id: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        End a trip

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          id: Specify the ID of the trip to be ended.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/skynet/trip/end",
            body=maybe_transform({"id": id}, trip_end_params.TripEndParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    trip_end_params.TripEndParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    def get_summary(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TripGetSummaryResponse:
        """
        Get summary of an ended trip

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/skynet/trip/{id}/summary",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    trip_get_summary_params.TripGetSummaryParams,
                ),
            ),
            cast_to=TripGetSummaryResponse,
        )

    def start(
        self,
        *,
        key: str,
        asset_id: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        attributes: object | NotGiven = NOT_GIVEN,
        custom_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        meta_data: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        stops: Iterable[trip_start_params.Stop] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TripStartResponse:
        """
        Add a new trip to the system with the provided data.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          asset_id: Specify the ID of the asset which is making this trip. The asset will be linked
              to this trip.

          cluster: the cluster of the region you want to use

          attributes: `attributes` can be used to store custom information about a trip in
              `key`:`value` format. Use `attributes` to add any useful information or context
              to your trips like the driver name, destination etc.

              Please note that the maximum number of `key`:`value` pairs that can be added to
              an `attributes` object is 100. Also, the overall size of `attributes` object
              should not exceed 65kb.

          custom_id: Set a unique ID for the new `trip`. If not provided, an ID will be automatically
              generated in UUID format. A valid `custom_id` can contain letters, numbers, “-”,
              & “\\__” only.

              Please note that the ID of a `trip` can not be changed once it is created.

          description: Add a custom description for the trip.

          meta_data: An JSON object to collect additional details about the trip. Use this property
              to add any custom information / context about the trip. The input will be passed
              on to the response as-is and can be used to display useful information on, for
              example, a UI app.

          name: Specify a name for the trip.

          stops: An array of objects to collect the details about all the stops that need to be
              made before the trip ends. Each object represents one stop.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/skynet/trip/start",
            body=maybe_transform(
                {
                    "asset_id": asset_id,
                    "attributes": attributes,
                    "custom_id": custom_id,
                    "description": description,
                    "meta_data": meta_data,
                    "name": name,
                    "stops": stops,
                },
                trip_start_params.TripStartParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    trip_start_params.TripStartParams,
                ),
            ),
            cast_to=TripStartResponse,
        )


class AsyncTripResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTripResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTripResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTripResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncTripResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TripRetrieveResponse:
        """
        Retrieves detailed information about a specific trip.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/skynet/trip/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    trip_retrieve_params.TripRetrieveParams,
                ),
            ),
            cast_to=TripRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        key: str,
        asset_id: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        attributes: object | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        meta_data: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        stops: Iterable[trip_update_params.Stop] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Updates the data of a specified trip with the provided data.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          asset_id: Use this param to update the ID of the asset which made this trip. Please be
              cautious when using this field as providing an ID other than what was provided
              at the time of starting the trip, will link a new asset to the trip and un-link
              the original asset, even if the trip is still active.

          cluster: the cluster of the region you want to use

          attributes: Use this field to update the `attributes` of the trip. Please note that when
              updating the `attributes` field, previously added information will be
              overwritten.

          description: Use this parameter to update the custom description of the trip.

          meta_data: Use this JSON object to update additional details about the trip. This property
              is used to add any custom information / context about the trip.

              Please note that updating the `meta_data` field will overwrite the previously
              added information.

          name: Use this property to update the name of the trip.

          stops: Use this object to update the details of the stops made during the trip. Each
              object represents a single stop.

              Please note that when updating this field, the new stops will overwrite any
              existing stops configured for the trip.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/skynet/trip/{id}",
            body=await async_maybe_transform(
                {
                    "asset_id": asset_id,
                    "attributes": attributes,
                    "description": description,
                    "meta_data": meta_data,
                    "name": name,
                    "stops": stops,
                },
                trip_update_params.TripUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    trip_update_params.TripUpdateParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    async def delete(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Deletes a specified trip from the system.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/skynet/trip/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    trip_delete_params.TripDeleteParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    async def end(
        self,
        *,
        key: str,
        id: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        End a trip

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          id: Specify the ID of the trip to be ended.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/skynet/trip/end",
            body=await async_maybe_transform({"id": id}, trip_end_params.TripEndParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    trip_end_params.TripEndParams,
                ),
            ),
            cast_to=SimpleResp,
        )

    async def get_summary(
        self,
        id: str,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TripGetSummaryResponse:
        """
        Get summary of an ended trip

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/skynet/trip/{id}/summary",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    trip_get_summary_params.TripGetSummaryParams,
                ),
            ),
            cast_to=TripGetSummaryResponse,
        )

    async def start(
        self,
        *,
        key: str,
        asset_id: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        attributes: object | NotGiven = NOT_GIVEN,
        custom_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        meta_data: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        stops: Iterable[trip_start_params.Stop] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TripStartResponse:
        """
        Add a new trip to the system with the provided data.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          asset_id: Specify the ID of the asset which is making this trip. The asset will be linked
              to this trip.

          cluster: the cluster of the region you want to use

          attributes: `attributes` can be used to store custom information about a trip in
              `key`:`value` format. Use `attributes` to add any useful information or context
              to your trips like the driver name, destination etc.

              Please note that the maximum number of `key`:`value` pairs that can be added to
              an `attributes` object is 100. Also, the overall size of `attributes` object
              should not exceed 65kb.

          custom_id: Set a unique ID for the new `trip`. If not provided, an ID will be automatically
              generated in UUID format. A valid `custom_id` can contain letters, numbers, “-”,
              & “\\__” only.

              Please note that the ID of a `trip` can not be changed once it is created.

          description: Add a custom description for the trip.

          meta_data: An JSON object to collect additional details about the trip. Use this property
              to add any custom information / context about the trip. The input will be passed
              on to the response as-is and can be used to display useful information on, for
              example, a UI app.

          name: Specify a name for the trip.

          stops: An array of objects to collect the details about all the stops that need to be
              made before the trip ends. Each object represents one stop.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/skynet/trip/start",
            body=await async_maybe_transform(
                {
                    "asset_id": asset_id,
                    "attributes": attributes,
                    "custom_id": custom_id,
                    "description": description,
                    "meta_data": meta_data,
                    "name": name,
                    "stops": stops,
                },
                trip_start_params.TripStartParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    trip_start_params.TripStartParams,
                ),
            ),
            cast_to=TripStartResponse,
        )


class TripResourceWithRawResponse:
    def __init__(self, trip: TripResource) -> None:
        self._trip = trip

        self.retrieve = to_raw_response_wrapper(
            trip.retrieve,
        )
        self.update = to_raw_response_wrapper(
            trip.update,
        )
        self.delete = to_raw_response_wrapper(
            trip.delete,
        )
        self.end = to_raw_response_wrapper(
            trip.end,
        )
        self.get_summary = to_raw_response_wrapper(
            trip.get_summary,
        )
        self.start = to_raw_response_wrapper(
            trip.start,
        )


class AsyncTripResourceWithRawResponse:
    def __init__(self, trip: AsyncTripResource) -> None:
        self._trip = trip

        self.retrieve = async_to_raw_response_wrapper(
            trip.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            trip.update,
        )
        self.delete = async_to_raw_response_wrapper(
            trip.delete,
        )
        self.end = async_to_raw_response_wrapper(
            trip.end,
        )
        self.get_summary = async_to_raw_response_wrapper(
            trip.get_summary,
        )
        self.start = async_to_raw_response_wrapper(
            trip.start,
        )


class TripResourceWithStreamingResponse:
    def __init__(self, trip: TripResource) -> None:
        self._trip = trip

        self.retrieve = to_streamed_response_wrapper(
            trip.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            trip.update,
        )
        self.delete = to_streamed_response_wrapper(
            trip.delete,
        )
        self.end = to_streamed_response_wrapper(
            trip.end,
        )
        self.get_summary = to_streamed_response_wrapper(
            trip.get_summary,
        )
        self.start = to_streamed_response_wrapper(
            trip.start,
        )


class AsyncTripResourceWithStreamingResponse:
    def __init__(self, trip: AsyncTripResource) -> None:
        self._trip = trip

        self.retrieve = async_to_streamed_response_wrapper(
            trip.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            trip.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            trip.delete,
        )
        self.end = async_to_streamed_response_wrapper(
            trip.end,
        )
        self.get_summary = async_to_streamed_response_wrapper(
            trip.get_summary,
        )
        self.start = async_to_streamed_response_wrapper(
            trip.start,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RestrictionsItemListParams"]


class RestrictionsItemListParams(TypedDict, total=False):
    max_lat: Required[float]

    max_lon: Required[float]

    min_lat: Required[float]

    min_lon: Required[float]

    group_id: float

    mode: Literal["0w", "1w", "2w", "3w", "4w", "6w"]

    restriction_type: Literal["turn", "parking", "fixedspeed", "maxspeed", "closure", "truck"]

    source: str

    state: Literal["enabled", "disabled", "deleted"]

    status: Literal["active", "inactive"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ConsoleSearchParams"]


class ConsoleSearchParams(TypedDict, total=False):
    query: Required[str]
    """string to be searched, will used to match name or id of geofence."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ...._models import BaseModel
from ..pagination import Pagination
from .track_location import TrackLocation

__all__ = [
    "LocationListResponse",
    "Data",
    "DataGeojson",
    "DataGeojsonGeometry",
    "DataSnappedPoint",
    "DataSnappedPointLocation",
]


class DataGeojsonGeometry(BaseModel):
    coordinates: Optional[List[float]] = None
    """
    An array of coordinates in the [longitude, latitude] format, representing the
    route geometry.
    """

    type: Optional[str] = None
    """Type of the geoJSON geometry."""


class DataGeojson(BaseModel):
    geometry: Optional[DataGeojsonGeometry] = None
    """An object with details of the geoJSON geometry of the route."""

    type: Optional[str] = None
    """Type of the geoJSON object."""


class DataSnappedPointLocation(BaseModel):
    lat: Optional[float] = None
    """Latitude of the snapped point."""

    lon: Optional[float] = None
    """Longitude of the snapped point."""


class DataSnappedPoint(BaseModel):
    bearing: Optional[str] = None
    """
    The bearing angle of the snapped point from the original tracked location, in
    radians. It indicates the direction of the snapped point.
    """

    distance: Optional[float] = None
    """
    The distance of the snapped point from the original tracked location, in meters.
    """

    location: Optional[DataSnappedPointLocation] = None
    """The latitude and longitude coordinates of the snapped point."""

    name: Optional[str] = None
    """The name of the street or road of the snapped point."""

    original_index: Optional[str] = FieldInfo(alias="originalIndex", default=None)
    """The index of the tracked location to which this snapped point corresponds to."""


class Data(BaseModel):
    distance: Optional[float] = None
    """
    Distance of the path, in meters, formed by connecting all tracked locations
    returned.

    Please note that `distance` is returned only when the `mapmatch` property of
    `correction` parameter is set to 1.
    """

    geojson: Optional[DataGeojson] = None
    """An object with geoJSON details of the route.

    It is returned only when the `mapmatch` property of the `correction` parameter
    is set to 1 and `geometry_type` is `geojson, otherwise it is not present in the
    response. The contents of this object follow the
    [geoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946).
    """

    geometry: Optional[List[str]] = None
    """Geometry of tracked locations in the requested format.

    It is returned only if the `mapmatch` property of the ‘correction’ parameter is
    set to 1.
    """

    list: Optional[List[TrackLocation]] = None
    """An array of objects with details of the tracked locations of the `asset`.

    Each object represents one tracked location.
    """

    page: Optional[Pagination] = None
    """An object with pagination details of the search results.

    Use this object to implement pagination in your application.
    """

    snapped_points: Optional[List[DataSnappedPoint]] = None
    """
    An array of objects with details about the snapped points for each of the
    tracked locations returned for the `asset`.

    Please note that this property is returned only when the `mapmatch` property of
    `correction` parameter is set to 1.
    """


class LocationListResponse(BaseModel):
    data: Optional[Data] = None

    message: Optional[str] = None
    """Displays the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types import (
    GeocodeRetrieveResponse,
    GeocodeBatchCreateResponse,
    GeocodeStructuredRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestGeocode:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: NextbillionSDK) -> None:
        geocode = client.geocode.retrieve(
            key="key=API_KEY",
            q="q=125, Berliner, berlin",
        )
        assert_matches_type(GeocodeRetrieveResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_with_all_params(self, client: NextbillionSDK) -> None:
        geocode = client.geocode.retrieve(
            key="key=API_KEY",
            q="q=125, Berliner, berlin",
            at="at=52.5308,13.3856",
            in_="in=countryCode:CAN,MEX,USA",
            lang="lang=en",
            limit=0,
        )
        assert_matches_type(GeocodeRetrieveResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: NextbillionSDK) -> None:
        response = client.geocode.with_raw_response.retrieve(
            key="key=API_KEY",
            q="q=125, Berliner, berlin",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geocode = response.parse()
        assert_matches_type(GeocodeRetrieveResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: NextbillionSDK) -> None:
        with client.geocode.with_streaming_response.retrieve(
            key="key=API_KEY",
            q="q=125, Berliner, berlin",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geocode = response.parse()
            assert_matches_type(GeocodeRetrieveResponse, geocode, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_batch_create(self, client: NextbillionSDK) -> None:
        geocode = client.geocode.batch_create(
            key="key=API_KEY",
            body=[{"q": '"q":"125, Berliner, berlin"'}],
        )
        assert_matches_type(GeocodeBatchCreateResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_batch_create(self, client: NextbillionSDK) -> None:
        response = client.geocode.with_raw_response.batch_create(
            key="key=API_KEY",
            body=[{"q": '"q":"125, Berliner, berlin"'}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geocode = response.parse()
        assert_matches_type(GeocodeBatchCreateResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_batch_create(self, client: NextbillionSDK) -> None:
        with client.geocode.with_streaming_response.batch_create(
            key="key=API_KEY",
            body=[{"q": '"q":"125, Berliner, berlin"'}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geocode = response.parse()
            assert_matches_type(GeocodeBatchCreateResponse, geocode, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_structured_retrieve(self, client: NextbillionSDK) -> None:
        geocode = client.geocode.structured_retrieve(
            country_code="countryCode",
            key="key=API_KEY",
        )
        assert_matches_type(GeocodeStructuredRetrieveResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_structured_retrieve_with_all_params(self, client: NextbillionSDK) -> None:
        geocode = client.geocode.structured_retrieve(
            country_code="countryCode",
            key="key=API_KEY",
            at="at=52.5308,13.3856",
            city="city",
            county="county",
            house_number="houseNumber",
            in_="in=circle:52.53,13.38;r=10000",
            limit=0,
            postal_code="postalCode",
            state="state",
            street="street",
        )
        assert_matches_type(GeocodeStructuredRetrieveResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_structured_retrieve(self, client: NextbillionSDK) -> None:
        response = client.geocode.with_raw_response.structured_retrieve(
            country_code="countryCode",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geocode = response.parse()
        assert_matches_type(GeocodeStructuredRetrieveResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_structured_retrieve(self, client: NextbillionSDK) -> None:
        with client.geocode.with_streaming_response.structured_retrieve(
            country_code="countryCode",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geocode = response.parse()
            assert_matches_type(GeocodeStructuredRetrieveResponse, geocode, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncGeocode:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        geocode = await async_client.geocode.retrieve(
            key="key=API_KEY",
            q="q=125, Berliner, berlin",
        )
        assert_matches_type(GeocodeRetrieveResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        geocode = await async_client.geocode.retrieve(
            key="key=API_KEY",
            q="q=125, Berliner, berlin",
            at="at=52.5308,13.3856",
            in_="in=countryCode:CAN,MEX,USA",
            lang="lang=en",
            limit=0,
        )
        assert_matches_type(GeocodeRetrieveResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geocode.with_raw_response.retrieve(
            key="key=API_KEY",
            q="q=125, Berliner, berlin",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geocode = await response.parse()
        assert_matches_type(GeocodeRetrieveResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geocode.with_streaming_response.retrieve(
            key="key=API_KEY",
            q="q=125, Berliner, berlin",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geocode = await response.parse()
            assert_matches_type(GeocodeRetrieveResponse, geocode, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_batch_create(self, async_client: AsyncNextbillionSDK) -> None:
        geocode = await async_client.geocode.batch_create(
            key="key=API_KEY",
            body=[{"q": '"q":"125, Berliner, berlin"'}],
        )
        assert_matches_type(GeocodeBatchCreateResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_batch_create(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geocode.with_raw_response.batch_create(
            key="key=API_KEY",
            body=[{"q": '"q":"125, Berliner, berlin"'}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geocode = await response.parse()
        assert_matches_type(GeocodeBatchCreateResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_batch_create(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geocode.with_streaming_response.batch_create(
            key="key=API_KEY",
            body=[{"q": '"q":"125, Berliner, berlin"'}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geocode = await response.parse()
            assert_matches_type(GeocodeBatchCreateResponse, geocode, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_structured_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        geocode = await async_client.geocode.structured_retrieve(
            country_code="countryCode",
            key="key=API_KEY",
        )
        assert_matches_type(GeocodeStructuredRetrieveResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_structured_retrieve_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        geocode = await async_client.geocode.structured_retrieve(
            country_code="countryCode",
            key="key=API_KEY",
            at="at=52.5308,13.3856",
            city="city",
            county="county",
            house_number="houseNumber",
            in_="in=circle:52.53,13.38;r=10000",
            limit=0,
            postal_code="postalCode",
            state="state",
            street="street",
        )
        assert_matches_type(GeocodeStructuredRetrieveResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_structured_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geocode.with_raw_response.structured_retrieve(
            country_code="countryCode",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        geocode = await response.parse()
        assert_matches_type(GeocodeStructuredRetrieveResponse, geocode, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_structured_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geocode.with_streaming_response.structured_retrieve(
            country_code="countryCode",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            geocode = await response.parse()
            assert_matches_type(GeocodeStructuredRetrieveResponse, geocode, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["MdmCreateDistanceMatrixResponse"]


class MdmCreateDistanceMatrixResponse(BaseModel):
    code: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages/codes
    are returned in case of errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """

    message: Optional[str] = None
    """Returns the error message in case a request fails.

    This field will not be present in the response, if a request is successfully
    submitted.
    """

    task_id: Optional[str] = None
    """
    A unique ID which can be used in the Asynchronous Distance Matrix GET method to
    retrieve the final result.
    """

    warning: Optional[List[str]] = None
    """Display the warnings, if any, for the given input parameters and values.

    In case there are no warnings then this field would not be present in the
    response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["ContactObject", "Category"]


class Category(BaseModel):
    id: Optional[str] = None
    """Identifier number for an associated category. For example: "900-9300-0000" """


class ContactObject(BaseModel):
    categories: Optional[List[Category]] = None
    """The list of place categories this contact refers to."""

    label: Optional[str] = None
    """
    Optional label for the contact string, such as "Customer Service" or "Pharmacy
    Fax".
    """

    value: Optional[str] = None
    """Contact information, as specified by the contact type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["StepDeleteParams"]


class StepDeleteParams(TypedDict, total=False):
    route_id: Required[Annotated[str, PropertyInfo(alias="routeID")]]

    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .contact_object import ContactObject

__all__ = ["Contacts"]


class Contacts(BaseModel):
    email: Optional[List[ContactObject]] = None

    fax: Optional[List[ContactObject]] = None

    mobile: Optional[List[ContactObject]] = None

    phone: Optional[List[ContactObject]] = None

    toll_free: Optional[List[ContactObject]] = FieldInfo(alias="tollFree", default=None)

    www: Optional[List[ContactObject]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .routing_response import RoutingResponse as RoutingResponse
from .route_create_params import RouteCreateParams as RouteCreateParams
from .route_create_response import RouteCreateResponse as RouteCreateResponse
from .route_redispatch_params import RouteRedispatchParams as RouteRedispatchParams
from .route_redispatch_response import RouteRedispatchResponse as RouteRedispatchResponse
from .document_template_list_params import DocumentTemplateListParams as DocumentTemplateListParams
from .document_template_create_params import DocumentTemplateCreateParams as DocumentTemplateCreateParams
from .document_template_delete_params import DocumentTemplateDeleteParams as DocumentTemplateDeleteParams
from .document_template_list_response import DocumentTemplateListResponse as DocumentTemplateListResponse
from .document_template_update_params import DocumentTemplateUpdateParams as DocumentTemplateUpdateParams
from .document_template_create_response import DocumentTemplateCreateResponse as DocumentTemplateCreateResponse
from .document_template_delete_response import DocumentTemplateDeleteResponse as DocumentTemplateDeleteResponse
from .document_template_retrieve_params import DocumentTemplateRetrieveParams as DocumentTemplateRetrieveParams
from .document_template_update_response import DocumentTemplateUpdateResponse as DocumentTemplateUpdateResponse
from .document_template_content_response import DocumentTemplateContentResponse as DocumentTemplateContentResponse
from .document_template_retrieve_response import DocumentTemplateRetrieveResponse as DocumentTemplateRetrieveResponse
from .document_template_content_request_param import (
    DocumentTemplateContentRequestParam as DocumentTemplateContentRequestParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["PlaceItemParam", "Geopoint", "Poi"]


class Geopoint(TypedDict, total=False):
    lat: float
    """This parameter represents the latitude value of the place."""

    lng: float
    """This parameter represents the longitude value of the place."""


class Poi(TypedDict, total=False):
    title: str
    """A title that describes the point of interest."""


class PlaceItemParam(TypedDict, total=False):
    address: str
    """
    This parameter represents the complete address of the place, including the
    street, city, state, postal code and country.
    """

    building: str
    """This parameter represents additional building information if applicable."""

    city: str
    """This parameter represents the city or town of the place."""

    country: str
    """This parameter represents the country of the place."""

    district: str
    """This parameter represents the district of the place."""

    geopoint: Geopoint
    """This parameter represents the geographical coordinates of the place.

    It includes the latitude and longitude values.
    """

    house: str
    """This parameter represents the house or building number of the place."""

    poi: Poi
    """This parameter represents a point of interest within the place.

    A Point of Interest (POI) refers to a specific location or area that is of
    interest to individuals for various reasons. It could be a landmark, tourist
    attraction, business, or any other location that people might find important or
    intriguing.
    """

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """This parameter represents the postal code or ZIP code of the place."""

    state: str
    """This parameter represents the state or region of the place."""

    street: str
    """This parameter represents the street name of the place."""

    sub_district: Annotated[str, PropertyInfo(alias="subDistrict")]
    """This parameter represents the sub-district or locality of the place."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .routing_response import RoutingResponse

__all__ = [
    "RouteRedispatchResponse",
    "Data",
    "DataCompletion",
    "DataDriver",
    "DataSteps",
    "DataStepsCompletion",
    "DataStepsMeta",
]


class DataCompletion(BaseModel):
    status: Optional[Literal["`scheduled`", "`completed`"]] = None
    """Returns the status of the route.

    It can take one of the following values - "scheduled", "completed".
    """


class DataDriver(BaseModel):
    id: Optional[str] = None
    """
    Returns the ID of the driver as specified in the
    [NextBillion.ai](http://NextBillion.ai) Cloud Console.
    """

    email: Optional[str] = None
    """
    Returns the email of the driver as specified in the
    [NextBillion.ai](http://NextBillion.ai) Cloud Console.
    """

    fullname: Optional[str] = None
    """
    Returns the full name of the driver as specified in
    [NextBillion.ai](http://NextBillion.ai) Cloud Console.
    """


class DataStepsCompletion(BaseModel):
    status: Optional[str] = None
    """Returns the status of the step.

    It can take one of the following values - "scheduled", "completed".
    """


class DataStepsMeta(BaseModel):
    customer_name: Optional[str] = None
    """Returns the customer name associated with the step.

    It can configured in the input request using the `metadata` attribute of the
    step.
    """

    customer_phone_number: Optional[str] = None
    """Returns the customer's phone number associated with the step.

    It can configured in the input request using the `metadata` attribute of the
    step.
    """

    instructions: Optional[str] = None
    """Returns the custom instructions to carry out while performing the task.

    These instructions can be provided at the time of configuring the step details
    in the input request.
    """


class DataSteps(BaseModel):
    id: Optional[str] = None
    """Returns the unique ID of the step."""

    address: Optional[str] = None
    """Returns the postal address where the step is executed.

    Its value is the same as that specified in the input request when configuring
    the step details.
    """

    arrival: Optional[int] = None
    """
    Returns the scheduled arrival time of the driver at the step as an UNIX
    timestamp, in seconds precision. It is the same as that specified in the input
    request while configuring the step details.

    The timestamp returned here is only for informative display on the driver's app
    and it does not impact or get affected by the route generated.
    """

    completion: Optional[DataStepsCompletion] = None
    """Returns the completion status of the step."""

    created_at: Optional[int] = None
    """
    Returns the UNIX timestamp, in seconds precision, at which this step was
    created.
    """

    document_snapshot: Optional[List[object]] = None
    """
    Returns the details of the document that was used for collecting the proof of
    completion for the step. In case no document template ID was provided for the
    given step, then a `null` value is returned. Each object represents a new field
    in the document.
    """

    duration: Optional[int] = None
    """Returns the duration for `layover` or `break` type steps."""

    location: Optional[List[float]] = None
    """Returns the location coordinates where the step is executed."""

    meta: Optional[DataStepsMeta] = None
    """
    An object returning custom details about the step that were configured in the
    input request while configuring the step details. The information returned here
    will be available for display on the Driver's app under step details.
    """

    short_id: Optional[str] = None
    """
    Returns a unique short ID of the step for easier referencing and displaying
    purposes.
    """

    type: Optional[str] = None
    """Returns the step type.

    It can belong to one of the following: `start`, `job` , `pickup`, `delivery`,
    `break`, `layover` , and `end`. For any given step, it would be the same as that
    specified in the input request while configuring the step details.
    """

    updated_at: Optional[int] = None
    """
    Returns the UNIX timestamp, in seconds precision, at which this step was last
    updated.
    """


class Data(BaseModel):
    id: Optional[str] = None
    """Returns the unique ID of the route."""

    completed_steps: Optional[int] = None
    """Returns the number of steps already completed in the route."""

    completion: Optional[DataCompletion] = None
    """Returns the completion status of the route."""

    created_at: Optional[int] = None
    """
    Returns the UNIX timestamp, in seconds precision, at which this route dispatch
    request was created.
    """

    distance: Optional[int] = None
    """
    Returns the total route distance, in meters, for informative display in the
    driver app. It is the same as the value provided for `distance` field in the
    input request.
    """

    document_snapshot: Optional[List[object]] = None
    """
    Returns the details of the document that was specified in the input for
    collecting the proof-of-completion for all steps in the dispatched routes. Each
    object represents a new field in the document.
    """

    driver: Optional[DataDriver] = None
    """An object returning the details of the driver to whom the route was dispatched."""

    ro_request_id: Optional[str] = None
    """Returns the route optimization request ID which was used to dispatch the route.

    An empty string is returned if the corresponding input was not provided.
    """

    routing: Optional[RoutingResponse] = None
    """
    An object returning the routing characteristics that are used to generate the
    route and turn-by-turn navigation steps for the dispatched route. The route and
    navigation steps are available when driver uses the in-app navigation.

    Please note the routing characteristics returned here are the same as those
    configured in the input request. The fields which were not specified in the
    input will be returned as blanks.
    """

    short_id: Optional[str] = None
    """
    Returns a shorter unique ID of the route for easier referencing and displaying
    purposes.
    """

    steps: Optional[DataSteps] = None

    total_steps: Optional[int] = None
    """Returns the total number of steps in the dispatched route."""

    updated_at: Optional[int] = None
    """
    Returns the UNIX timestamp, in seconds precision, at which this route dispatch
    request was updated.
    """

    vehicle_id: Optional[str] = None
    """Returns the ID of the vehicle to which the route was dispatched.

    The vehicle ID returned here is the same as the one used in the route
    optimization request for the given vehicle. An empty string is returned if the
    `ro_request_id` was not provided in the input.
    """


class RouteRedispatchResponse(BaseModel):
    data: Optional[Data] = None
    """An array of objects containing the details of each step in the dispatched route.

    Each object represents a single step.
    """

    message: Optional[str] = None
    """Returns the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[int] = None
    """Returns the status code of the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["RouteStepCompletionMode"]

RouteStepCompletionMode: TypeAlias = Literal["`manual`", "`geofence`", "`geofence_manual_fallback`"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .search import (
    SearchResource,
    AsyncSearchResource,
    SearchResourceWithRawResponse,
    AsyncSearchResourceWithRawResponse,
    SearchResourceWithStreamingResponse,
    AsyncSearchResourceWithStreamingResponse,
)
from .polygon import (
    PolygonResource,
    AsyncPolygonResource,
    PolygonResourceWithRawResponse,
    AsyncPolygonResourceWithRawResponse,
    PolygonResourceWithStreamingResponse,
    AsyncPolygonResourceWithStreamingResponse,
)

__all__ = [
    "PolygonResource",
    "AsyncPolygonResource",
    "PolygonResourceWithRawResponse",
    "AsyncPolygonResourceWithRawResponse",
    "PolygonResourceWithStreamingResponse",
    "AsyncPolygonResourceWithStreamingResponse",
    "SearchResource",
    "AsyncSearchResource",
    "SearchResourceWithRawResponse",
    "AsyncSearchResourceWithRawResponse",
    "SearchResourceWithStreamingResponse",
    "AsyncSearchResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types.geofence import (
    ConsoleSearchResponse,
    ConsolePreviewResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestConsole:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_preview(self, client: NextbillionSDK) -> None:
        console = client.geofence.console.preview(
            type="`circle`",
        )
        assert_matches_type(ConsolePreviewResponse, console, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_preview_with_all_params(self, client: NextbillionSDK) -> None:
        console = client.geofence.console.preview(
            type="`circle`",
            circle={
                "center": {
                    "lat": 0,
                    "lon": 0,
                },
                "radius": 0,
            },
            custom_id="custom_id",
            isochrone={
                "coordinates": '"coordinates": "13.25805884,77.91083661"',
                "contours_meter": 0,
                "contours_minute": 0,
                "denoise": 0,
                "departure_time": 0,
                "mode": "`car`",
            },
            meta_data='{\n  "country": "USA",\n  "state": "California"\n}',
            name='"name":"Los Angeles Downtown"',
            polygon={
                "geojson": {
                    "coordinates": [[0]],
                    "type": "type",
                }
            },
            tags=['"tags":["tags_1", "O69Am2Y4KL8q5Y5JuD-Fy-tdtEpkTRQo_ZYIK7"]'],
        )
        assert_matches_type(ConsolePreviewResponse, console, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_preview(self, client: NextbillionSDK) -> None:
        response = client.geofence.console.with_raw_response.preview(
            type="`circle`",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        console = response.parse()
        assert_matches_type(ConsolePreviewResponse, console, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_preview(self, client: NextbillionSDK) -> None:
        with client.geofence.console.with_streaming_response.preview(
            type="`circle`",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            console = response.parse()
            assert_matches_type(ConsolePreviewResponse, console, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_search(self, client: NextbillionSDK) -> None:
        console = client.geofence.console.search(
            query="query",
        )
        assert_matches_type(ConsoleSearchResponse, console, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_search(self, client: NextbillionSDK) -> None:
        response = client.geofence.console.with_raw_response.search(
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        console = response.parse()
        assert_matches_type(ConsoleSearchResponse, console, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_search(self, client: NextbillionSDK) -> None:
        with client.geofence.console.with_streaming_response.search(
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            console = response.parse()
            assert_matches_type(ConsoleSearchResponse, console, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncConsole:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_preview(self, async_client: AsyncNextbillionSDK) -> None:
        console = await async_client.geofence.console.preview(
            type="`circle`",
        )
        assert_matches_type(ConsolePreviewResponse, console, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_preview_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        console = await async_client.geofence.console.preview(
            type="`circle`",
            circle={
                "center": {
                    "lat": 0,
                    "lon": 0,
                },
                "radius": 0,
            },
            custom_id="custom_id",
            isochrone={
                "coordinates": '"coordinates": "13.25805884,77.91083661"',
                "contours_meter": 0,
                "contours_minute": 0,
                "denoise": 0,
                "departure_time": 0,
                "mode": "`car`",
            },
            meta_data='{\n  "country": "USA",\n  "state": "California"\n}',
            name='"name":"Los Angeles Downtown"',
            polygon={
                "geojson": {
                    "coordinates": [[0]],
                    "type": "type",
                }
            },
            tags=['"tags":["tags_1", "O69Am2Y4KL8q5Y5JuD-Fy-tdtEpkTRQo_ZYIK7"]'],
        )
        assert_matches_type(ConsolePreviewResponse, console, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_preview(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geofence.console.with_raw_response.preview(
            type="`circle`",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        console = await response.parse()
        assert_matches_type(ConsolePreviewResponse, console, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_preview(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geofence.console.with_streaming_response.preview(
            type="`circle`",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            console = await response.parse()
            assert_matches_type(ConsolePreviewResponse, console, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_search(self, async_client: AsyncNextbillionSDK) -> None:
        console = await async_client.geofence.console.search(
            query="query",
        )
        assert_matches_type(ConsoleSearchResponse, console, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_search(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.geofence.console.with_raw_response.search(
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        console = await response.parse()
        assert_matches_type(ConsoleSearchResponse, console, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_search(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.geofence.console.with_streaming_response.search(
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            console = await response.parse()
            assert_matches_type(ConsoleSearchResponse, console, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["PlaceCreateResponse"]


class PlaceCreateResponse(BaseModel):
    doc_id: Optional[str] = FieldInfo(alias="docId", default=None)
    """A unique NextBillion DocID will be created for the POI.

    Use this ID to search this place through the “Get Place” method, to update
    attributes or ‘status’ through the “Update Place” method or delete it using the
    “Delete Place” method.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from .trip import (
    TripResource,
    AsyncTripResource,
    TripResourceWithRawResponse,
    AsyncTripResourceWithRawResponse,
    TripResourceWithStreamingResponse,
    AsyncTripResourceWithStreamingResponse,
)
from .config import (
    ConfigResource,
    AsyncConfigResource,
    ConfigResourceWithRawResponse,
    AsyncConfigResourceWithRawResponse,
    ConfigResourceWithStreamingResponse,
    AsyncConfigResourceWithStreamingResponse,
)
from ...types import skynet_subscribe_params
from .monitor import (
    MonitorResource,
    AsyncMonitorResource,
    MonitorResourceWithRawResponse,
    AsyncMonitorResourceWithRawResponse,
    MonitorResourceWithStreamingResponse,
    AsyncMonitorResourceWithStreamingResponse,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .asset.asset import (
    AssetResource,
    AsyncAssetResource,
    AssetResourceWithRawResponse,
    AsyncAssetResourceWithRawResponse,
    AssetResourceWithStreamingResponse,
    AsyncAssetResourceWithStreamingResponse,
)
from .search.search import (
    SearchResource,
    AsyncSearchResource,
    SearchResourceWithRawResponse,
    AsyncSearchResourceWithRawResponse,
    SearchResourceWithStreamingResponse,
    AsyncSearchResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from .namespaced_apikeys import (
    NamespacedApikeysResource,
    AsyncNamespacedApikeysResource,
    NamespacedApikeysResourceWithRawResponse,
    AsyncNamespacedApikeysResourceWithRawResponse,
    NamespacedApikeysResourceWithStreamingResponse,
    AsyncNamespacedApikeysResourceWithStreamingResponse,
)
from ...types.skynet_subscribe_response import SkynetSubscribeResponse

__all__ = ["SkynetResource", "AsyncSkynetResource"]


class SkynetResource(SyncAPIResource):
    @cached_property
    def asset(self) -> AssetResource:
        return AssetResource(self._client)

    @cached_property
    def monitor(self) -> MonitorResource:
        return MonitorResource(self._client)

    @cached_property
    def trip(self) -> TripResource:
        return TripResource(self._client)

    @cached_property
    def namespaced_apikeys(self) -> NamespacedApikeysResource:
        return NamespacedApikeysResource(self._client)

    @cached_property
    def config(self) -> ConfigResource:
        return ConfigResource(self._client)

    @cached_property
    def search(self) -> SearchResource:
        return SearchResource(self._client)

    @cached_property
    def with_raw_response(self) -> SkynetResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return SkynetResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SkynetResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return SkynetResourceWithStreamingResponse(self)

    def subscribe(
        self,
        *,
        action: Literal["`TRIP_SUBSCRIBE`", "`TRIP_UNSUBSCRIBE`", "`HEARTBEAT`"],
        id: str | NotGiven = NOT_GIVEN,
        params: skynet_subscribe_params.Params | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SkynetSubscribeResponse:
        """
        POST Action

        Args:
          action: Specify the behavior that needs to be achieved for the subscription. Following
              values are accepted:

              - TRIP_SUBSCRIBE: Enable a trip subscription.
              - TRIP_UNSUBSCRIBE: Unsubscribe from a trip
              - HEARTBEAT: Enable heartbeat mechanism for a web-socket connection. The action
                message need to be sent at a frequency higher than every 5 mins to keep the
                connection alive. Alternatively, users can chose to respond to the ping frame
                sent by web socket server to keep the connection alive. Refer to
                [connection details](https://188--nbai-docs-stg.netlify.app/docs/tracking/api/live-tracking-api#connect-to-web-socket-server)
                for more details.

          id: Specify a custom ID for the subscription. It can be used to reference a given
              subscription in the downstream applications / systems.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/skynet/subscribe",
            body=maybe_transform(
                {
                    "action": action,
                    "id": id,
                    "params": params,
                },
                skynet_subscribe_params.SkynetSubscribeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SkynetSubscribeResponse,
        )


class AsyncSkynetResource(AsyncAPIResource):
    @cached_property
    def asset(self) -> AsyncAssetResource:
        return AsyncAssetResource(self._client)

    @cached_property
    def monitor(self) -> AsyncMonitorResource:
        return AsyncMonitorResource(self._client)

    @cached_property
    def trip(self) -> AsyncTripResource:
        return AsyncTripResource(self._client)

    @cached_property
    def namespaced_apikeys(self) -> AsyncNamespacedApikeysResource:
        return AsyncNamespacedApikeysResource(self._client)

    @cached_property
    def config(self) -> AsyncConfigResource:
        return AsyncConfigResource(self._client)

    @cached_property
    def search(self) -> AsyncSearchResource:
        return AsyncSearchResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSkynetResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSkynetResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSkynetResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncSkynetResourceWithStreamingResponse(self)

    async def subscribe(
        self,
        *,
        action: Literal["`TRIP_SUBSCRIBE`", "`TRIP_UNSUBSCRIBE`", "`HEARTBEAT`"],
        id: str | NotGiven = NOT_GIVEN,
        params: skynet_subscribe_params.Params | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SkynetSubscribeResponse:
        """
        POST Action

        Args:
          action: Specify the behavior that needs to be achieved for the subscription. Following
              values are accepted:

              - TRIP_SUBSCRIBE: Enable a trip subscription.
              - TRIP_UNSUBSCRIBE: Unsubscribe from a trip
              - HEARTBEAT: Enable heartbeat mechanism for a web-socket connection. The action
                message need to be sent at a frequency higher than every 5 mins to keep the
                connection alive. Alternatively, users can chose to respond to the ping frame
                sent by web socket server to keep the connection alive. Refer to
                [connection details](https://188--nbai-docs-stg.netlify.app/docs/tracking/api/live-tracking-api#connect-to-web-socket-server)
                for more details.

          id: Specify a custom ID for the subscription. It can be used to reference a given
              subscription in the downstream applications / systems.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/skynet/subscribe",
            body=await async_maybe_transform(
                {
                    "action": action,
                    "id": id,
                    "params": params,
                },
                skynet_subscribe_params.SkynetSubscribeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SkynetSubscribeResponse,
        )


class SkynetResourceWithRawResponse:
    def __init__(self, skynet: SkynetResource) -> None:
        self._skynet = skynet

        self.subscribe = to_raw_response_wrapper(
            skynet.subscribe,
        )

    @cached_property
    def asset(self) -> AssetResourceWithRawResponse:
        return AssetResourceWithRawResponse(self._skynet.asset)

    @cached_property
    def monitor(self) -> MonitorResourceWithRawResponse:
        return MonitorResourceWithRawResponse(self._skynet.monitor)

    @cached_property
    def trip(self) -> TripResourceWithRawResponse:
        return TripResourceWithRawResponse(self._skynet.trip)

    @cached_property
    def namespaced_apikeys(self) -> NamespacedApikeysResourceWithRawResponse:
        return NamespacedApikeysResourceWithRawResponse(self._skynet.namespaced_apikeys)

    @cached_property
    def config(self) -> ConfigResourceWithRawResponse:
        return ConfigResourceWithRawResponse(self._skynet.config)

    @cached_property
    def search(self) -> SearchResourceWithRawResponse:
        return SearchResourceWithRawResponse(self._skynet.search)


class AsyncSkynetResourceWithRawResponse:
    def __init__(self, skynet: AsyncSkynetResource) -> None:
        self._skynet = skynet

        self.subscribe = async_to_raw_response_wrapper(
            skynet.subscribe,
        )

    @cached_property
    def asset(self) -> AsyncAssetResourceWithRawResponse:
        return AsyncAssetResourceWithRawResponse(self._skynet.asset)

    @cached_property
    def monitor(self) -> AsyncMonitorResourceWithRawResponse:
        return AsyncMonitorResourceWithRawResponse(self._skynet.monitor)

    @cached_property
    def trip(self) -> AsyncTripResourceWithRawResponse:
        return AsyncTripResourceWithRawResponse(self._skynet.trip)

    @cached_property
    def namespaced_apikeys(self) -> AsyncNamespacedApikeysResourceWithRawResponse:
        return AsyncNamespacedApikeysResourceWithRawResponse(self._skynet.namespaced_apikeys)

    @cached_property
    def config(self) -> AsyncConfigResourceWithRawResponse:
        return AsyncConfigResourceWithRawResponse(self._skynet.config)

    @cached_property
    def search(self) -> AsyncSearchResourceWithRawResponse:
        return AsyncSearchResourceWithRawResponse(self._skynet.search)


class SkynetResourceWithStreamingResponse:
    def __init__(self, skynet: SkynetResource) -> None:
        self._skynet = skynet

        self.subscribe = to_streamed_response_wrapper(
            skynet.subscribe,
        )

    @cached_property
    def asset(self) -> AssetResourceWithStreamingResponse:
        return AssetResourceWithStreamingResponse(self._skynet.asset)

    @cached_property
    def monitor(self) -> MonitorResourceWithStreamingResponse:
        return MonitorResourceWithStreamingResponse(self._skynet.monitor)

    @cached_property
    def trip(self) -> TripResourceWithStreamingResponse:
        return TripResourceWithStreamingResponse(self._skynet.trip)

    @cached_property
    def namespaced_apikeys(self) -> NamespacedApikeysResourceWithStreamingResponse:
        return NamespacedApikeysResourceWithStreamingResponse(self._skynet.namespaced_apikeys)

    @cached_property
    def config(self) -> ConfigResourceWithStreamingResponse:
        return ConfigResourceWithStreamingResponse(self._skynet.config)

    @cached_property
    def search(self) -> SearchResourceWithStreamingResponse:
        return SearchResourceWithStreamingResponse(self._skynet.search)


class AsyncSkynetResourceWithStreamingResponse:
    def __init__(self, skynet: AsyncSkynetResource) -> None:
        self._skynet = skynet

        self.subscribe = async_to_streamed_response_wrapper(
            skynet.subscribe,
        )

    @cached_property
    def asset(self) -> AsyncAssetResourceWithStreamingResponse:
        return AsyncAssetResourceWithStreamingResponse(self._skynet.asset)

    @cached_property
    def monitor(self) -> AsyncMonitorResourceWithStreamingResponse:
        return AsyncMonitorResourceWithStreamingResponse(self._skynet.monitor)

    @cached_property
    def trip(self) -> AsyncTripResourceWithStreamingResponse:
        return AsyncTripResourceWithStreamingResponse(self._skynet.trip)

    @cached_property
    def namespaced_apikeys(self) -> AsyncNamespacedApikeysResourceWithStreamingResponse:
        return AsyncNamespacedApikeysResourceWithStreamingResponse(self._skynet.namespaced_apikeys)

    @cached_property
    def config(self) -> AsyncConfigResourceWithStreamingResponse:
        return AsyncConfigResourceWithStreamingResponse(self._skynet.config)

    @cached_property
    def search(self) -> AsyncSearchResourceWithStreamingResponse:
        return AsyncSearchResourceWithStreamingResponse(self._skynet.search)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["Categories"]


class Categories(BaseModel):
    id: Optional[str] = None
    """Identifier number for an associated category."""

    name: Optional[str] = None
    """Name of the place category in the result item language."""

    primary: Optional[str] = None
    """Whether or not it is a primary category.

    This field is visible only when the value is 'true'.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types import PostResponse
from nextbillion_sdk.types.optimization import V2RetrieveResultResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestV2:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_result(self, client: NextbillionSDK) -> None:
        v2 = client.optimization.v2.retrieve_result(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(V2RetrieveResultResponse, v2, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_result(self, client: NextbillionSDK) -> None:
        response = client.optimization.v2.with_raw_response.retrieve_result(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        v2 = response.parse()
        assert_matches_type(V2RetrieveResultResponse, v2, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_result(self, client: NextbillionSDK) -> None:
        with client.optimization.v2.with_streaming_response.retrieve_result(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            v2 = response.parse()
            assert_matches_type(V2RetrieveResultResponse, v2, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_submit(self, client: NextbillionSDK) -> None:
        v2 = client.optimization.v2.submit(
            key="key=API_KEY",
            locations={"location": ["string"]},
            vehicles=[
                {
                    "id": "id",
                    "location": {},
                }
            ],
        )
        assert_matches_type(PostResponse, v2, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_submit_with_all_params(self, client: NextbillionSDK) -> None:
        v2 = client.optimization.v2.submit(
            key="key=API_KEY",
            locations={
                "location": ["string"],
                "id": 0,
                "approaches": ["`unrestricted`"],
            },
            vehicles=[
                {
                    "id": "id",
                    "location": {
                        "lat": 0,
                        "lng": 0,
                    },
                    "attributes": '"attributes":{\n    "driver_rating": "4.0",\n    "trip_types": "premium"\n  }',
                    "priority": 0,
                    "remaining_waypoints": [
                        {
                            "lat": -90,
                            "lon": -180,
                        }
                    ],
                }
            ],
            cost_matrix=[[0]],
            depots=[
                {
                    "id": '"id":"depot 1"',
                    "location_index": 0,
                    "description": "“description”:”Los_Angeles_depot”",
                    "service": 0,
                    "time_windows": [[0]],
                }
            ],
            description='"description": "Sample Optimization"',
            distance_matrix=[[0]],
            duration_matrix=[[0]],
            existing_solution_id="existing_solution_id",
            jobs=[
                {
                    "id": '"id":"Job 1"',
                    "location_index": 0,
                    "delivery": [0],
                    "depot_ids": ["string"],
                    "description": "description",
                    "follow_lifo_order": True,
                    "incompatible_load_types": ["string"],
                    "joint_order": 0,
                    "load_types": ["string"],
                    "max_visit_lateness": 0,
                    "metadata": '{\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                    "outsourcing_cost": 0,
                    "pickup": [0],
                    "priority": 0,
                    "revenue": 0,
                    "sequence_order": 0,
                    "service": 0,
                    "setup": 0,
                    "skills": [1],
                    "time_windows": [[0]],
                    "volume": {
                        "alignment": "`strict`",
                        "depth": 0,
                        "height": 0,
                        "width": 0,
                    },
                    "zones": [0],
                }
            ],
            options={
                "constraint": {
                    "max_activity_waiting_time": 0,
                    "max_vehicle_overtime": 0,
                    "max_visit_lateness": 0,
                },
                "grouping": {
                    "order_grouping": {"grouping_diameter": 0},
                    "proximity_factor": 0,
                    "route_grouping": {
                        "penalty_factor": 0,
                        "zone_diameter": 0,
                        "zone_source": "`system_generated`",
                    },
                },
                "objective": {
                    "allow_early_arrival": True,
                    "custom": {
                        "type": "`min`",
                        "value": "`vehicles`",
                    },
                    "minimise_num_depots": True,
                    "solver_mode": "`flexible`",
                    "solving_time_limit": 0,
                    "travel_cost": "`duration`",
                },
                "routing": {
                    "allow": ["taxi"],
                    "avoid": ["`toll`"],
                    "context": "`avgspeed`",
                    "cross_border": True,
                    "disable_cache": True,
                    "hazmat_type": ["`general`"],
                    "mode": "`car`",
                    "profiles": '"profiles":{\n    "mini-van":{\n        "mode": "car",\n        "avoid":["highway, toll"]\n        },\n    "trailer":{\n        "mode": "truck",\n        "truck_weight":12000,\n        "truck_size":"200, 210, 600",\n        "hazmat_type": ["general", "harmful_to_water"]\n        }\n    }\n',
                    "traffic_timestamp": 0,
                    "truck_axle_load": 0,
                    "truck_size": '"truck_size":"200,210,600"',
                    "truck_weight": 0,
                },
            },
            relations=[
                {
                    "steps": [
                        {
                            "type": "`start`",
                            "id": '"id":"Job 1"',
                        }
                    ],
                    "type": "`in_same_route`",
                    "id": 0,
                    "max_duration": 0,
                    "min_duration": 0,
                    "vehicle": '"vehicle": "Vehicle 10"',
                }
            ],
            shipments=[
                {
                    "delivery": {
                        "id": '"id":"Shipment Delivery 1"',
                        "location_index": 0,
                        "description": "description",
                        "max_visit_lateness": 0,
                        "metadata": '{\n  "notes": "dropoff at the patio",\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                        "sequence_order": 0,
                        "service": 0,
                        "setup": 0,
                        "time_windows": [[0]],
                    },
                    "pickup": {
                        "id": '"id": "Shipment Pickup 1"',
                        "location_index": 0,
                        "description": "description",
                        "max_visit_lateness": 0,
                        "metadata": '{\n  "notes": "involves fragile items",\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                        "sequence_order": 0,
                        "service": 0,
                        "setup": 0,
                        "time_windows": [[0]],
                    },
                    "amount": [0],
                    "follow_lifo_order": True,
                    "incompatible_load_types": ["string"],
                    "joint_order": 0,
                    "load_types": ["string"],
                    "max_time_in_vehicle": 0,
                    "outsourcing_cost": 0,
                    "priority": 0,
                    "revenue": 0,
                    "skills": [0],
                    "volume": {
                        "alignment": "`strict`",
                        "depth": 0,
                        "height": 0,
                        "width": 0,
                    },
                    "zones": [0],
                }
            ],
            solution=[
                {
                    "cost": 0,
                    "steps": [
                        {
                            "id": '"id": "Job 10"',
                            "arrival": 0,
                            "type": "`start`",
                            "description": "description",
                            "distance": 0,
                            "duration": 0,
                            "load": [0],
                            "location": [0],
                            "location_index": 0,
                            "service": 0,
                            "setup": 0,
                            "waiting_time": 0,
                        }
                    ],
                    "vehicle": "vehicle",
                    "delivery": [0],
                    "description": "description",
                    "distance": 0,
                    "duration": 0,
                    "geometry": '"geometry": "}ebGgcsxRE?CuDOYCYAG???"',
                    "pickup": [0],
                    "priority": 0,
                    "service": 0,
                    "setup": 0,
                    "waiting_time": 0,
                }
            ],
            unassigned={
                "jobs": ["string"],
                "shipments": [["string"]],
            },
            zones=[
                {
                    "id": 0,
                    "geofence_id": "geofence_id",
                    "geometry": {
                        "coordinates": [[0]],
                        "description": "description",
                        "type": "`Polygon`",
                    },
                }
            ],
        )
        assert_matches_type(PostResponse, v2, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_submit(self, client: NextbillionSDK) -> None:
        response = client.optimization.v2.with_raw_response.submit(
            key="key=API_KEY",
            locations={"location": ["string"]},
            vehicles=[
                {
                    "id": "id",
                    "location": {},
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        v2 = response.parse()
        assert_matches_type(PostResponse, v2, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_submit(self, client: NextbillionSDK) -> None:
        with client.optimization.v2.with_streaming_response.submit(
            key="key=API_KEY",
            locations={"location": ["string"]},
            vehicles=[
                {
                    "id": "id",
                    "location": {},
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            v2 = response.parse()
            assert_matches_type(PostResponse, v2, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncV2:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_result(self, async_client: AsyncNextbillionSDK) -> None:
        v2 = await async_client.optimization.v2.retrieve_result(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(V2RetrieveResultResponse, v2, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_result(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.optimization.v2.with_raw_response.retrieve_result(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        v2 = await response.parse()
        assert_matches_type(V2RetrieveResultResponse, v2, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_result(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.optimization.v2.with_streaming_response.retrieve_result(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            v2 = await response.parse()
            assert_matches_type(V2RetrieveResultResponse, v2, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_submit(self, async_client: AsyncNextbillionSDK) -> None:
        v2 = await async_client.optimization.v2.submit(
            key="key=API_KEY",
            locations={"location": ["string"]},
            vehicles=[
                {
                    "id": "id",
                    "location": {},
                }
            ],
        )
        assert_matches_type(PostResponse, v2, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_submit_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        v2 = await async_client.optimization.v2.submit(
            key="key=API_KEY",
            locations={
                "location": ["string"],
                "id": 0,
                "approaches": ["`unrestricted`"],
            },
            vehicles=[
                {
                    "id": "id",
                    "location": {
                        "lat": 0,
                        "lng": 0,
                    },
                    "attributes": '"attributes":{\n    "driver_rating": "4.0",\n    "trip_types": "premium"\n  }',
                    "priority": 0,
                    "remaining_waypoints": [
                        {
                            "lat": -90,
                            "lon": -180,
                        }
                    ],
                }
            ],
            cost_matrix=[[0]],
            depots=[
                {
                    "id": '"id":"depot 1"',
                    "location_index": 0,
                    "description": "“description”:”Los_Angeles_depot”",
                    "service": 0,
                    "time_windows": [[0]],
                }
            ],
            description='"description": "Sample Optimization"',
            distance_matrix=[[0]],
            duration_matrix=[[0]],
            existing_solution_id="existing_solution_id",
            jobs=[
                {
                    "id": '"id":"Job 1"',
                    "location_index": 0,
                    "delivery": [0],
                    "depot_ids": ["string"],
                    "description": "description",
                    "follow_lifo_order": True,
                    "incompatible_load_types": ["string"],
                    "joint_order": 0,
                    "load_types": ["string"],
                    "max_visit_lateness": 0,
                    "metadata": '{\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                    "outsourcing_cost": 0,
                    "pickup": [0],
                    "priority": 0,
                    "revenue": 0,
                    "sequence_order": 0,
                    "service": 0,
                    "setup": 0,
                    "skills": [1],
                    "time_windows": [[0]],
                    "volume": {
                        "alignment": "`strict`",
                        "depth": 0,
                        "height": 0,
                        "width": 0,
                    },
                    "zones": [0],
                }
            ],
            options={
                "constraint": {
                    "max_activity_waiting_time": 0,
                    "max_vehicle_overtime": 0,
                    "max_visit_lateness": 0,
                },
                "grouping": {
                    "order_grouping": {"grouping_diameter": 0},
                    "proximity_factor": 0,
                    "route_grouping": {
                        "penalty_factor": 0,
                        "zone_diameter": 0,
                        "zone_source": "`system_generated`",
                    },
                },
                "objective": {
                    "allow_early_arrival": True,
                    "custom": {
                        "type": "`min`",
                        "value": "`vehicles`",
                    },
                    "minimise_num_depots": True,
                    "solver_mode": "`flexible`",
                    "solving_time_limit": 0,
                    "travel_cost": "`duration`",
                },
                "routing": {
                    "allow": ["taxi"],
                    "avoid": ["`toll`"],
                    "context": "`avgspeed`",
                    "cross_border": True,
                    "disable_cache": True,
                    "hazmat_type": ["`general`"],
                    "mode": "`car`",
                    "profiles": '"profiles":{\n    "mini-van":{\n        "mode": "car",\n        "avoid":["highway, toll"]\n        },\n    "trailer":{\n        "mode": "truck",\n        "truck_weight":12000,\n        "truck_size":"200, 210, 600",\n        "hazmat_type": ["general", "harmful_to_water"]\n        }\n    }\n',
                    "traffic_timestamp": 0,
                    "truck_axle_load": 0,
                    "truck_size": '"truck_size":"200,210,600"',
                    "truck_weight": 0,
                },
            },
            relations=[
                {
                    "steps": [
                        {
                            "type": "`start`",
                            "id": '"id":"Job 1"',
                        }
                    ],
                    "type": "`in_same_route`",
                    "id": 0,
                    "max_duration": 0,
                    "min_duration": 0,
                    "vehicle": '"vehicle": "Vehicle 10"',
                }
            ],
            shipments=[
                {
                    "delivery": {
                        "id": '"id":"Shipment Delivery 1"',
                        "location_index": 0,
                        "description": "description",
                        "max_visit_lateness": 0,
                        "metadata": '{\n  "notes": "dropoff at the patio",\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                        "sequence_order": 0,
                        "service": 0,
                        "setup": 0,
                        "time_windows": [[0]],
                    },
                    "pickup": {
                        "id": '"id": "Shipment Pickup 1"',
                        "location_index": 0,
                        "description": "description",
                        "max_visit_lateness": 0,
                        "metadata": '{\n  "notes": "involves fragile items",\n  "contact": "212-456-7890",\n  "metaId": 1234\n}',
                        "sequence_order": 0,
                        "service": 0,
                        "setup": 0,
                        "time_windows": [[0]],
                    },
                    "amount": [0],
                    "follow_lifo_order": True,
                    "incompatible_load_types": ["string"],
                    "joint_order": 0,
                    "load_types": ["string"],
                    "max_time_in_vehicle": 0,
                    "outsourcing_cost": 0,
                    "priority": 0,
                    "revenue": 0,
                    "skills": [0],
                    "volume": {
                        "alignment": "`strict`",
                        "depth": 0,
                        "height": 0,
                        "width": 0,
                    },
                    "zones": [0],
                }
            ],
            solution=[
                {
                    "cost": 0,
                    "steps": [
                        {
                            "id": '"id": "Job 10"',
                            "arrival": 0,
                            "type": "`start`",
                            "description": "description",
                            "distance": 0,
                            "duration": 0,
                            "load": [0],
                            "location": [0],
                            "location_index": 0,
                            "service": 0,
                            "setup": 0,
                            "waiting_time": 0,
                        }
                    ],
                    "vehicle": "vehicle",
                    "delivery": [0],
                    "description": "description",
                    "distance": 0,
                    "duration": 0,
                    "geometry": '"geometry": "}ebGgcsxRE?CuDOYCYAG???"',
                    "pickup": [0],
                    "priority": 0,
                    "service": 0,
                    "setup": 0,
                    "waiting_time": 0,
                }
            ],
            unassigned={
                "jobs": ["string"],
                "shipments": [["string"]],
            },
            zones=[
                {
                    "id": 0,
                    "geofence_id": "geofence_id",
                    "geometry": {
                        "coordinates": [[0]],
                        "description": "description",
                        "type": "`Polygon`",
                    },
                }
            ],
        )
        assert_matches_type(PostResponse, v2, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_submit(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.optimization.v2.with_raw_response.submit(
            key="key=API_KEY",
            locations={"location": ["string"]},
            vehicles=[
                {
                    "id": "id",
                    "location": {},
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        v2 = await response.parse()
        assert_matches_type(PostResponse, v2, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_submit(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.optimization.v2.with_streaming_response.submit(
            key="key=API_KEY",
            locations={"location": ["string"]},
            vehicles=[
                {
                    "id": "id",
                    "location": {},
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            v2 = await response.parse()
            assert_matches_type(PostResponse, v2, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import postalcode_retrieve_coordinates_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.postalcode_retrieve_coordinates_response import PostalcodeRetrieveCoordinatesResponse

__all__ = ["PostalcodeResource", "AsyncPostalcodeResource"]


class PostalcodeResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PostalcodeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PostalcodeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PostalcodeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return PostalcodeResourceWithStreamingResponse(self)

    def retrieve_coordinates(
        self,
        *,
        key: str,
        at: postalcode_retrieve_coordinates_params.At | NotGiven = NOT_GIVEN,
        country: str | NotGiven = NOT_GIVEN,
        format: Literal["`geojson`"] | NotGiven = NOT_GIVEN,
        postalcode: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PostalcodeRetrieveCoordinatesResponse:
        """
        Retrieve coordinates by postal code

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          at: Location coordinates that you want to get the postal code of. If not providing
              `postalcode` in the request, `at` becomes mandatory. Please note that only 1
              point can be requested. [See this example](#note).

          country: Country containing the postal code or the location. It is mandatory if
              `postalcode` is provided in the request. [See this example](#note).

              Please check the [API Query Limits](#api-query-limits) section below for a list
              of the countries covered by the Geocode Postcode API. Users can provide either
              the name or the alpha-2/3 code as per the
              [ISO 3166-1 standard](https://en.wikipedia.org/wiki/ISO_3166-1) of a country
              covered by the API as input for this parameter.

          format: Specify the format in which the boundary details of the post code will be
              returned. When specified, the boundary details will be returned in the `geojson`
              format. When not specified, the boundary details are returned in general format.

          postalcode: Provide the postal code for which the information is needed. At least one of
              (`postalcode` + `country`) or `at` needs to be provided. Please note that only 1
              postal code can be requested. [See this example](#note).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/postalcode",
            body=maybe_transform(
                {
                    "at": at,
                    "country": country,
                    "format": format,
                    "postalcode": postalcode,
                },
                postalcode_retrieve_coordinates_params.PostalcodeRetrieveCoordinatesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"key": key}, postalcode_retrieve_coordinates_params.PostalcodeRetrieveCoordinatesParams
                ),
            ),
            cast_to=PostalcodeRetrieveCoordinatesResponse,
        )


class AsyncPostalcodeResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPostalcodeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPostalcodeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPostalcodeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncPostalcodeResourceWithStreamingResponse(self)

    async def retrieve_coordinates(
        self,
        *,
        key: str,
        at: postalcode_retrieve_coordinates_params.At | NotGiven = NOT_GIVEN,
        country: str | NotGiven = NOT_GIVEN,
        format: Literal["`geojson`"] | NotGiven = NOT_GIVEN,
        postalcode: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PostalcodeRetrieveCoordinatesResponse:
        """
        Retrieve coordinates by postal code

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          at: Location coordinates that you want to get the postal code of. If not providing
              `postalcode` in the request, `at` becomes mandatory. Please note that only 1
              point can be requested. [See this example](#note).

          country: Country containing the postal code or the location. It is mandatory if
              `postalcode` is provided in the request. [See this example](#note).

              Please check the [API Query Limits](#api-query-limits) section below for a list
              of the countries covered by the Geocode Postcode API. Users can provide either
              the name or the alpha-2/3 code as per the
              [ISO 3166-1 standard](https://en.wikipedia.org/wiki/ISO_3166-1) of a country
              covered by the API as input for this parameter.

          format: Specify the format in which the boundary details of the post code will be
              returned. When specified, the boundary details will be returned in the `geojson`
              format. When not specified, the boundary details are returned in general format.

          postalcode: Provide the postal code for which the information is needed. At least one of
              (`postalcode` + `country`) or `at` needs to be provided. Please note that only 1
              postal code can be requested. [See this example](#note).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/postalcode",
            body=await async_maybe_transform(
                {
                    "at": at,
                    "country": country,
                    "format": format,
                    "postalcode": postalcode,
                },
                postalcode_retrieve_coordinates_params.PostalcodeRetrieveCoordinatesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"key": key}, postalcode_retrieve_coordinates_params.PostalcodeRetrieveCoordinatesParams
                ),
            ),
            cast_to=PostalcodeRetrieveCoordinatesResponse,
        )


class PostalcodeResourceWithRawResponse:
    def __init__(self, postalcode: PostalcodeResource) -> None:
        self._postalcode = postalcode

        self.retrieve_coordinates = to_raw_response_wrapper(
            postalcode.retrieve_coordinates,
        )


class AsyncPostalcodeResourceWithRawResponse:
    def __init__(self, postalcode: AsyncPostalcodeResource) -> None:
        self._postalcode = postalcode

        self.retrieve_coordinates = async_to_raw_response_wrapper(
            postalcode.retrieve_coordinates,
        )


class PostalcodeResourceWithStreamingResponse:
    def __init__(self, postalcode: PostalcodeResource) -> None:
        self._postalcode = postalcode

        self.retrieve_coordinates = to_streamed_response_wrapper(
            postalcode.retrieve_coordinates,
        )


class AsyncPostalcodeResourceWithStreamingResponse:
    def __init__(self, postalcode: AsyncPostalcodeResource) -> None:
        self._postalcode = postalcode

        self.retrieve_coordinates = async_to_streamed_response_wrapper(
            postalcode.retrieve_coordinates,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options

__all__ = ["MapResource", "AsyncMapResource"]


class MapResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MapResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return MapResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MapResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return MapResourceWithStreamingResponse(self)

    def create_segment(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """Road Segments"""
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/map/segments",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncMapResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMapResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMapResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMapResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncMapResourceWithStreamingResponse(self)

    async def create_segment(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """Road Segments"""
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/map/segments",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class MapResourceWithRawResponse:
    def __init__(self, map: MapResource) -> None:
        self._map = map

        self.create_segment = to_raw_response_wrapper(
            map.create_segment,
        )


class AsyncMapResourceWithRawResponse:
    def __init__(self, map: AsyncMapResource) -> None:
        self._map = map

        self.create_segment = async_to_raw_response_wrapper(
            map.create_segment,
        )


class MapResourceWithStreamingResponse:
    def __init__(self, map: MapResource) -> None:
        self._map = map

        self.create_segment = to_streamed_response_wrapper(
            map.create_segment,
        )


class AsyncMapResourceWithStreamingResponse:
    def __init__(self, map: AsyncMapResource) -> None:
        self._map = map

        self.create_segment = async_to_streamed_response_wrapper(
            map.create_segment,
        )
